@@ -1,7 +1,7 @@
 /* DASDISUP.C   (c) Copyright Roger Bowler, 1999-2007                */
 /*              Hercules DASD Utilities: IEHIOSUP                    */
 
-// $Id: dasdisup.c,v 1.30 2007/06/23 00:04:08 ivan Exp $
+// $Id: dasdisup.c,v 1.31 2008/11/04 04:50:46 fish Exp $
 
 /*-------------------------------------------------------------------*/
 /* This program performs the IEHIOSUP function of OS/360.            */
@@ -14,6 +14,9 @@
 /*-------------------------------------------------------------------*/
 
 // $Log: dasdisup.c,v $
+// Revision 1.31  2008/11/04 04:50:46  fish
+// Ensure consistent utility startup
+//
 // Revision 1.30  2007/06/23 00:04:08  ivan
 // Update copyright notices to include current year (2007)
 //
@@ -623,21 +626,7 @@ CIFBLK         *cif;                    /* CKD image file descriptor */
 MEMINFO        *memtab;                 /* -> Member info array      */
 int             nmem = 0;               /* Number of array entries   */
 
-#if defined(ENABLE_NLS)
-    setlocale(LC_ALL, "");
-    bindtextdomain(PACKAGE, HERC_LOCALEDIR);
-    textdomain(PACKAGE);
-#endif
-
-#ifdef EXTERNALGUI
-    if (argc >= 1 && strncmp(argv[argc-1],"EXTERNALGUI",11) == 0)
-    {
-        extgui = 1;
-        argc--;
-        setvbuf(stderr, NULL, _IONBF, 0);
-        setvbuf(stdout, NULL, _IONBF, 0);
-    }
-#endif /*EXTERNALGUI*/
+    INITIALIZE_UTILITY("dasdisup");
 
     /* Display the program identification message */
     display_version (stderr,
