@@ -1,7 +1,7 @@
 /* DASDPDSU.C   (c) Copyright Roger Bowler, 1999-2007                */
 /*              Hercules DASD Utilities: PDS unloader                */
 
-// $Id: dasdpdsu.c,v 1.27 2007/06/23 00:04:08 ivan Exp $
+// $Id: dasdpdsu.c,v 1.28 2008/11/04 04:50:46 fish Exp $
 
 /*-------------------------------------------------------------------*/
 /* This program unloads members of a partitioned dataset from        */
@@ -19,6 +19,9 @@
 /*-------------------------------------------------------------------*/
 
 // $Log: dasdpdsu.c,v $
+// Revision 1.28  2008/11/04 04:50:46  fish
+// Ensure consistent utility startup
+//
 // Revision 1.27  2007/06/23 00:04:08  ivan
 // Update copyright notices to include current year (2007)
 //
@@ -243,15 +246,7 @@ BYTE           *blkptr;                 /* -> PDS directory block    */
 BYTE            dirblk[256];            /* Copy of directory block   */
 CIFBLK         *cif;                    /* CKD image file descriptor */
 
-#ifdef EXTERNALGUI
-    if (argc >= 1 && strncmp(argv[argc-1],"EXTERNALGUI",11) == 0)
-    {
-        extgui = 1;
-        argc--;
-        setvbuf(stderr, NULL, _IONBF, 0);
-        setvbuf(stdout, NULL, _IONBF, 0);
-    }
-#endif /*EXTERNALGUI*/
+    INITIALIZE_UTILITY("dasdpdsu");
 
     /* Display the program identification message */
     display_version (stderr, "Hercules PDS unload program ", FALSE);
