@@ -1,7 +1,7 @@
 /* DASDTAB.C    (c) Copyright Roger Bowler, 1999-2007                */
 /*              Hercules Supported DASD definitions                  */
 
-// $Id: dasdtab.c,v 1.37 2007/06/23 00:04:08 ivan Exp $
+// $Id: dasdtab.c,v 1.40 2008/11/02 21:45:07 rbowler Exp $
 
 /*-------------------------------------------------------------------*/
 /* This module contains the tables that define the attributes of     */
@@ -15,6 +15,15 @@
 /*-------------------------------------------------------------------*/
 
 // $Log: dasdtab.c,v $
+// Revision 1.40  2008/11/02 21:45:07  rbowler
+// Correction to dasdtab entries for 3375 and 3380
+//
+// Revision 1.39  2008/10/27 15:10:32  rbowler
+// Revert to 3990 as default control unit for 3390 devices
+//
+// Revision 1.38  2008/02/13 00:30:22  gsmith
+// Fix 3880 SNSID length (thanks De!)
+//
 // Revision 1.37  2007/06/23 00:04:08  ivan
 // Update copyright notices to include current year (2007)
 //
@@ -72,36 +81,39 @@ static CKDDEV ckdtab[] = {
 
  {"3350",      0x3350,0x00,0x20,0x00,  555,5,30,19254,19069, 185,19254,128,0x0000,-1,82,185,   0,   0,  0,0,"3830"},
  {"3350-1",    0x3350,0x00,0x20,0x00,  555,5,30,19254,19069, 185,19254,128,0x0000,-1,82,185,   0,   0,  0,0,"3830"},
+/* name         type model clas code prime a hd    r0    r1 har0   len sec    rps  f f1  f2   f3   f4 f5 f6  cu */
 
- {"3375",      0x3375,0x02,0x20,0x0e,  959,3,12,36000,35616, 832,36000,196,0x5007, 1, 0x20,0x01,0x08,0x00,0x0a,0x00,"3880"},
- {"3375-1",    0x3375,0x02,0x20,0x0e,  959,3,12,36000,35616, 832,36000,196,0x5007, 1, 0x20,0x01,0x08,0x00,0x0a,0x00,"3880"},
+ {"3375",      0x3375,0x02,0x20,0x0e,  959,3,12,36000,35616, 832,36000,196,0x5007, 1, 32,384,160,   0,  0,0,"3880"},
+ {"3375-1",    0x3375,0x02,0x20,0x0e,  959,3,12,36000,35616, 832,36000,196,0x5007, 1, 32,384,160,   0,  0,0,"3880"},
  
- {"3380",      0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
- {"3380-1",    0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
- {"3380-A",    0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
- {"3380-B",    0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
- {"3380-D",    0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
- {"3380-J",    0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
- {"3380-2",    0x3380,0x0a,0x20,0x0e, 1770,2,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
- {"3380-E",    0x3380,0x0a,0x20,0x0e, 1770,2,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
- {"3380-3",    0x3380,0x1e,0x20,0x0e, 2655,3,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
- {"3380-K",    0x3380,0x1e,0x20,0x0e, 2655,3,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
- {"EMC3380K+", 0x3380,0x1e,0x20,0x0e, 3339,3,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
- {"EMC3380K++",0x3380,0x1e,0x20,0x0e, 3993,3,15,47988,47476,1088,47968,222,0x0000, 1, 0x20,0x01,0xec,0x00,0xec,0x00,"3880"},
+ {"3380",      0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
+ {"3380-1",    0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
+ {"3380-A",    0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
+ {"3380-B",    0x3380,0x02,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"}, 
+ {"3380-D",    0x3380,0x06,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
+ {"3380-J",    0x3380,0x16,0x20,0x0e,  885,1,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
+ {"3380-2",    0x3380,0x0a,0x20,0x0e, 1770,2,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
+ {"3380-E",    0x3380,0x0a,0x20,0x0e, 1770,2,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
+ {"3380-3",    0x3380,0x1e,0x20,0x0e, 2655,3,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
+ {"3380-K",    0x3380,0x1e,0x20,0x0e, 2655,3,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
+ {"EMC3380K+", 0x3380,0x1e,0x20,0x0e, 3339,3,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
+ {"EMC3380K++",0x3380,0x1e,0x20,0x0e, 3993,3,15,47988,47476,1088,47968,222,0x5007, 1, 32,492,236,   0,  0,0,"3880"},
+/* name         type model clas code prime a hd    r0    r1 har0   len sec    rps  f f1  f2   f3   f4 f5 f6  cu */
 
- {"3390",      0x3390,0x02,0x20,0x26, 1113,1,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990-6"},
- {"3390-1",    0x3390,0x02,0x20,0x26, 1113,1,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990-6"},
- {"3390-2",    0x3390,0x06,0x20,0x27, 2226,1,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990-6"},
- {"3390-3",    0x3390,0x0a,0x20,0x24, 3339,1,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990-6"},
- {"3390-9",    0x3390,0x0c,0x20,0x32,10017,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990-6"},
- {"3390-27",   0x3390,0x0c,0x20,0x32,32760,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990-6"},
- {"3390-J",    0x3390,0x0c,0x20,0x32,32760,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990-6"},
- {"3390-54",   0x3390,0x0c,0x20,0x32,65520,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990-6"},
- {"3390-JJ",   0x3390,0x0c,0x20,0x32,65520,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990-6"},
+ {"3390",      0x3390,0x02,0x20,0x26, 1113,1,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
+ {"3390-1",    0x3390,0x02,0x20,0x26, 1113,1,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
+ {"3390-2",    0x3390,0x06,0x20,0x27, 2226,1,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
+ {"3390-3",    0x3390,0x0a,0x20,0x24, 3339,1,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
+ {"3390-9",    0x3390,0x0c,0x20,0x32,10017,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
+ {"3390-27",   0x3390,0x0c,0x20,0x32,32760,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
+ {"3390-J",    0x3390,0x0c,0x20,0x32,32760,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
+ {"3390-54",   0x3390,0x0c,0x20,0x32,65520,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
+ {"3390-JJ",   0x3390,0x0c,0x20,0x32,65520,3,15,57326,56664,1428,58786,224,0x7708, 2, 34,19,   9,   6,116,6,"3990"},
 
  {"9345",      0x9345,0x04,0x20,0x04, 1440,0,15,48174,46456,1184,48280,213,0x8b07, 2, 34,18,   7,   6,116,6,"9343"},
  {"9345-1",    0x9345,0x04,0x20,0x04, 1440,0,15,48174,46456,1184,48280,213,0x8b07, 2, 34,18,   7,   6,116,6,"9343"},
  {"9345-2",    0x9345,0x04,0x20,0x04, 2156,0,15,48174,46456,1184,48280,213,0x8b07, 2, 34,18,   7,   6,116,6,"9343"}
+/* name         type model clas code prime a hd    r0    r1 har0   len sec    rps  f f1  f2   f3   f4 f5 f6  cu */
 } ;
 #define CKDDEV_NUM (sizeof(ckdtab)/CKDDEV_SIZE)
 
@@ -240,9 +252,16 @@ int len;
     store_fw (devid +32, cu->ciw7);
     store_fw (devid +36, cu->ciw8);
 
-    /* Calculate length */
-    for (len = 40; fetch_fw(devid + len-4) == 0; len -= 4);
-    len = len < 12 ? 12 : len;
+    /* Set length */
+    if (cu->ciw8 != 0) len = 40;
+    else if (cu->ciw7 != 0) len = 36;
+    else if (cu->ciw6 != 0) len = 32;
+    else if (cu->ciw5 != 0) len = 28;
+    else if (cu->ciw4 != 0) len = 24;
+    else if (cu->ciw3 != 0) len = 20;
+    else if (cu->ciw2 != 0) len = 16;
+    else if (cu->ciw1 != 0) len = 12;
+    else len = 7;
 
     return len;
 }
@@ -272,14 +291,21 @@ int altcyls;                            /* Number alternate cyls     */
     devchar[16] = (BYTE)(ckd->sectors);         // Number of sectors
     store_hw(devchar+18, ckd->len);             // Track length
     store_hw(devchar+20, ckd->har0);            // Length of HA and R0
-    if (ckd->formula > 0)
+    if (ckd->formula == 1)
+    {
+        devchar[22] = (BYTE)(ckd->formula);     // Track capacity formula
+        devchar[23] = (BYTE)(ckd->f1);          // Factor F1
+        store_hw(devchar+24, ckd->f2);          // Factor F2
+        store_hw(devchar+26, ckd->f3);          // Factor F3
+    }
+    else if (ckd->formula == 2)
     {
         devchar[22] = (BYTE)(ckd->formula);     // Track capacity formula
         devchar[23] = (BYTE)(ckd->f1);          // Factor F1
         devchar[24] = (BYTE)(ckd->f2);          // Factor F2
-        devchar[25] = (BYTE)(ckd->f3);          // (F2F3)
-        devchar[26] = (BYTE)(ckd->f4);          // Factor F3
-        devchar[27] = (BYTE)(ckd->f5);          // (F4F5)
+        devchar[25] = (BYTE)(ckd->f3);          // Factor F3
+        devchar[26] = (BYTE)(ckd->f4);          // Factor F4
+        devchar[27] = (BYTE)(ckd->f5);          // Factor F5
     }
     if (altcyls > 0)
     {
