@@ -1,7 +1,7 @@
 /* FBADASD.C    (c) Copyright Roger Bowler, 1999-2007                */
 /*              ESA/390 FBA Direct Access Storage Device Handler     */
 
-// $Id: fbadasd.c,v 1.47 2007/06/23 00:04:09 ivan Exp $
+// $Id: fbadasd.c,v 1.48 2007/11/21 22:54:14 fish Exp $
 
 /*-------------------------------------------------------------------*/
 /* This module contains device handling functions for emulated       */
@@ -14,6 +14,9 @@
 /*-------------------------------------------------------------------*/
 
 // $Log: fbadasd.c,v $
+// Revision 1.48  2007/11/21 22:54:14  fish
+// Use new BEGIN_DEVICE_CLASS_QUERY macro
+//
 // Revision 1.47  2007/06/23 00:04:09  ivan
 // Update copyright notices to include current year (2007)
 //
@@ -332,7 +335,8 @@ void fbadasd_query_device (DEVBLK *dev, char **class,
                 int buflen, char *buffer)
 {
 
-    *class = "DASD";
+    BEGIN_DEVICE_CLASS_QUERY( "DASD", dev, class, buflen, buffer );
+
     snprintf (buffer, buflen, "%s [%lld,%d]",
             dev->filename,
             (long long)dev->fbaorigin, dev->fbanumblk);
