@@ -5,9 +5,16 @@
 /* compile option -DNO_ASM_BYTESWAP will expand 'C' code             */
 /* otherwise Intel (486+) assember will be generated                 */
 
-// $Id: hbyteswp.h,v 1.10 2006/12/08 09:43:21 jj Exp $
+// $Id: hbyteswp.h,v 1.12 2009/01/08 01:59:20 jmaynard Exp $
 //
 // $Log: hbyteswp.h,v $
+// Revision 1.12  2009/01/08 01:59:20  jmaynard
+// Got the test for 64-but NO_ASM_BYTESWAP backwards. Fixed.
+//
+// Revision 1.11  2009/01/07 18:00:13  jmaynard
+// Added x86_64 assembler swap assists for those systems that don't have
+// byteswap.h in the library.
+//
 // Revision 1.10  2006/12/08 09:43:21  jj
 // Add CVS message log
 //
@@ -35,13 +42,21 @@
 
     static __inline__ uint16_t (ATTR_REGPARM(1) bswap_16)(uint16_t x)
     {
+#if defined(__x86_64__)
+      __asm__("xchgb %b0,%h0" : "=Q" (x) :  "0" (x));
+#else
       __asm__("xchgb %b0,%h0" : "=q" (x) :  "0" (x));
+#endif
       return x;
     }
 
     static __inline__ uint32_t (ATTR_REGPARM(1) bswap_32)(uint32_t x)
     {
+#if defined(__x86_64__)
+      __asm__("bswapl %0" : "=r" (x) : "0" (x));
+#else
       __asm__("bswap %0" : "=r" (x) : "0" (x));
+#endif
       return x;
     }
 
@@ -83,7 +98,8 @@
 
 #else // !defined( _MSVC_ )
 
-  #define bswap_64(_x) \
+  #if defined(NO_ASM_BYTESWAP)
+    #define bswap_64(_x) \
           ( ((U64)((_x) & 0xFF00000000000000ULL) >> 56) \
           | ((U64)((_x) & 0x00FF000000000000ULL) >> 40) \
           | ((U64)((_x) & 0x0000FF0000000000ULL) >> 24) \
@@ -92,6 +108,31 @@
           | ((U64)((_x) & 0x0000000000FF0000ULL) << 24) \
           | ((U64)((_x) & 0x000000000000FF00ULL) << 40) \
           | ((U64)((_x) & 0x00000000000000FFULL) << 56) )
+  #else
+    static __inline__ uint64_t (ATTR_REGPARM(1) bswap_64)(uint64_t x)
+    {
+    #if defined(__x86_64__)
+      __asm__("bswapq %0" : "=r" (x) : "0" (x));
+      return x;
+    #else // swap the two words after byteswapping them
+      union
+      {
+        struct
+        {
+          uint32_t high,low;
+        } words;
+        uint64_t quad;
+      } value;
+      
+      value.quad=x;
+      __asm__("bswap %0" : "=r" (value.words.high) : "0" (value.words.high));
+      __asm__("bswap %0" : "=r" (value.words.low) : "0" (value.words.low));
+      __asm__("xchgl %0,%1" : "=r" (value.words.high), "=r" (value.words.low) :
+              "0" (value.words.high), "1" (value.words.low));
+      return value.quad;
+    #endif // defined(__x86_64__)
+    }
+  #endif // defined(NO_ASM_BYTESWAP)
 
 #endif // defined( _MSVC_ )
 
