@@ -7,9 +7,15 @@
 //      The <config.h> header and other required headers are
 //      presumed to have already been #included ahead of it...
 
-// $Id: hconsts.h,v 1.8 2007/03/20 22:23:33 gsmith Exp $
+// $Id: hconsts.h,v 1.10 2008/05/28 16:36:17 fish Exp $
 //
 // $Log: hconsts.h,v $
+// Revision 1.10  2008/05/28 16:36:17  fish
+// #define PATH_SEP constant
+//
+// Revision 1.9  2008/03/16 00:04:37  rbowler
+// Replace ACC_ARMODE by USE_ARMODE for LPTEA
+//
 // Revision 1.8  2007/03/20 22:23:33  gsmith
 // Redefine ACC_ and ACCTYPE_ macros
 //
@@ -29,8 +35,16 @@
 /* Miscellaneous system related constants we could be missing...     */
 /*-------------------------------------------------------------------*/
 
-#ifndef   MAX_PATH
-  #define MAX_PATH  PATH_MAX
+#ifndef     MAX_PATH
+  #define   MAX_PATH          PATH_MAX
+#endif
+
+#ifndef     PATH_SEP
+  #ifdef     _MSVC_
+    #define PATH_SEP          "\\"
+  #else
+    #define PATH_SEP          "/"
+  #endif
 #endif
 
 #if defined( _MSVC_ )
@@ -250,7 +264,6 @@
 #define ACC_PTE            0x0200          /* Return page table entry*/
 #define ACC_LPTEA          0x0400          /* Esame page table entry */
 #define ACC_SPECIAL_ART    0x0800          /* Used by BSG            */
-#define ACC_ARMODE         0x1000          /* Used by LPTEA          */
 
 #define ACCTYPE_HW         0               /* Hardware access        */
 #define ACCTYPE_INSTFETCH  ACC_READ        /* Instruction fetch      */
@@ -273,6 +286,8 @@
 #define USE_PRIMARY_SPACE       (-3)    /* Primary space virtual     */
 #define USE_SECONDARY_SPACE     (-4)    /* Secondary space virtual   */
 #define USE_HOME_SPACE          (-5)    /* Home space virtual        */
+#define USE_ARMODE              16      /* OR with access register
+                                           number to force AR mode   */
 
 /* Interception codes used by longjmp/SIE */
 #define SIE_NO_INTERCEPT        (-1)    /* Continue (after pgmint)   */
