@@ -1,9 +1,12 @@
 /* HDTEQ.C      (c) Copyright Jan Jaeger, 2003-2007                  */
 /*              Hercules Dynamic Loader                              */
 
-// $Id: hdteq.c,v 1.15 2007/06/23 00:04:10 ivan Exp $
+// $Id: hdteq.c,v 1.16 2008/12/28 15:30:09 jj Exp $
 //
 // $Log: hdteq.c,v $
+// Revision 1.16  2008/12/28 15:30:09  jj
+// SYSG and SYSA mods
+//
 // Revision 1.15  2007/06/23 00:04:10  ivan
 // Update copyright notices to include current year (2007)
 //
@@ -44,6 +47,7 @@ static DTEQ dteq[] = {
     { "1052",   "3270"  },
     { "3215",   "3270"  },
     { "3287",   "3270"  },
+    { "SYSG",   "3270"  },
 
     { "1052-C", "1052c" },
     { "3215-C", "1052c" },
