@@ -9,9 +9,12 @@
 || ----------------------------------------------------------------------------
 */
 
-// $Id: hetget.c,v 1.25 2007/06/23 00:04:10 ivan Exp $
+// $Id: hetget.c,v 1.26 2008/11/04 04:50:46 fish Exp $
 //
 // $Log: hetget.c,v $
+// Revision 1.26  2008/11/04 04:50:46  fish
+// Ensure consistent utility startup
+//
 // Revision 1.25  2007/06/23 00:04:10  ivan
 // Update copyright notices to include current year (2007)
 //
@@ -619,15 +622,7 @@ main( int argc, char *argv[] )
     int rc;
     int i;
 
-#ifdef EXTERNALGUI
-    if (argc >= 1 && strncmp(argv[argc-1],"EXTERNALGUI",11) == 0)
-    {
-        extgui = 1;
-        argc--;
-        setvbuf(stderr, NULL, _IONBF, 0);
-        setvbuf(stdout, NULL, _IONBF, 0);
-    }
-#endif /*EXTERNALGUI*/
+    INITIALIZE_UTILITY("hetget");
 
     /* Display the program identification message */
     display_version (stderr, "Hercules HET extract files program ", FALSE);
