@@ -1,6 +1,20 @@
-// $Id: hsys.c,v 1.6 2006/12/08 09:43:28 jj Exp $
+// $Id: hsys.c,v 1.10 2009/01/02 19:21:51 jj Exp $
 //
 // $Log: hsys.c,v $
+// Revision 1.10  2009/01/02 19:21:51  jj
+// DVD-RAM IPL
+// RAMSAVE
+// SYSG Integrated 3270 console fixes
+//
+// Revision 1.9  2008/12/24 15:42:14  jj
+// Add debug entry point for sclp event masks
+//
+// Revision 1.8  2008/12/22 13:10:22  jj
+// Add sclp debug entry points
+//
+// Revision 1.7  2008/02/12 08:42:15  fish
+// dyngui tweaks: new def devlist fmt, new debug_cd_cmd hook
+//
 // Revision 1.6  2006/12/08 09:43:28  jj
 // Add CVS message log
 //
@@ -29,12 +43,14 @@ DLL_EXPORT void  (*daemon_task) (void);
 DLL_EXPORT int   (*config_command) (int argc, char *argv[], char *cmdline);
 DLL_EXPORT int   (*system_command) (int argc, char *argv[], char *cmdline);
 DLL_EXPORT void *(*debug_cpu_state) (REGS *);
+DLL_EXPORT void  (*debug_cd_cmd) (char *);
 DLL_EXPORT void *(*debug_device_state) (DEVBLK *);
 DLL_EXPORT void *(*debug_program_interrupt) (REGS *, int);
 DLL_EXPORT void *(*debug_diagnose) (U32, int, int, REGS *);
 DLL_EXPORT void *(*debug_iucv) (int, VADR, REGS *);
 DLL_EXPORT void *(*debug_sclp_unknown_command) (U32, void *, REGS *);
 DLL_EXPORT void *(*debug_sclp_unknown_event) (void *, void *, REGS *);
+DLL_EXPORT void *(*debug_sclp_unknown_event_mask) (void *, void *, REGS *);
 DLL_EXPORT void *(*debug_sclp_event_data) (void *, void *, REGS *);
 DLL_EXPORT void *(*debug_chsc_unknown_request) (void *, void *, REGS *);
 DLL_EXPORT void *(*debug_watchdog_signal) (REGS *);
