@@ -2,9 +2,21 @@
 /*   HTYPES.H             Hercules typedefs...                       */
 /*-------------------------------------------------------------------*/
 
-// $Id: htypes.h,v 1.8 2006/12/08 09:43:28 jj Exp $
+// $Id: htypes.h,v 1.11 2008/07/08 05:35:51 fish Exp $
 //
 // $Log: htypes.h,v $
+// Revision 1.11  2008/07/08 05:35:51  fish
+// AUTOMOUNT redesign: support +allowed/-disallowed dirs
+// and create associated 'automount' panel command - Fish
+//
+// Revision 1.10  2008/06/22 05:54:30  fish
+// Fix print-formatting issue (mostly in tape modules)
+// that can sometimes, in certain circumstances,
+// cause herc to crash.  (%8.8lx --> I32_FMTX, etc)
+//
+// Revision 1.9  2008/02/19 17:18:36  rbowler
+// Missing u_int8_t causes crypto compile errors on Solaris
+//
 // Revision 1.8  2006/12/08 09:43:28  jj
 // Add CVS message log
 //
@@ -27,6 +39,17 @@
   #endif
 #endif
 
+#ifndef HAVE_U_INT8_T
+  #ifdef HAVE_INTTYPES_H
+    typedef  uint8_t   u_int8_t;
+    typedef  uint16_t  u_int16_t;
+    typedef  uint32_t  u_int32_t;
+    typedef  uint64_t  u_int64_t;
+  #else
+    #error Unable to define u_intNN_t data types
+  #endif
+#endif
+
 typedef  int8_t     S8;         // signed 8-bits
 typedef  int16_t    S16;        // signed 16-bits
 typedef  int32_t    S32;        // signed 32-bits
@@ -63,22 +86,40 @@ typedef  uint8_t    QWORD[16];  // unsigned quadword   (16 bytes)
   #define  I16_FMT                  "h"
   #define  I32_FMT                  ""
   #define  I64_FMT                  "l"
-#else // defined(SIZEOF_LONG) && SIZEOF_LONG == 4
+#else // !defined(SIZEOF_LONG) || SIZEOF_LONG != 8
   #define  I16_FMT                  "h"
   #define  I32_FMT                  ""
   #define  I64_FMT                  "ll"
 #endif
 
+#define  I16_FMTx           "%4.4" I16_FMT "x"
+#define  I32_FMTx           "%8.8" I32_FMT "x"
+#define  I64_FMTx         "%16.16" I64_FMT "x"
+
+#define  I16_FMTX           "%4.4" I16_FMT "X"
+#define  I32_FMTX           "%8.8" I32_FMT "X"
+#define  I64_FMTX         "%16.16" I64_FMT "X"
+
 #if defined(SIZEOF_LONG) && SIZEOF_LONG == 8
- #define UINT_PTR_FMT             I64_FMT
+ #define UINT_PTR_FMT              I64_FMT
  #define      PTR_FMTx    "%16.16" I64_FMT "x"
  #define      PTR_FMTX    "%16.16" I64_FMT "X"
-#else // defined(SIZEOF_LONG) && SIZEOF_LONG == 4
- #define UINT_PTR_FMT             I32_FMT
+#else // !defined(SIZEOF_LONG) || SIZEOF_LONG != 8
+ #define UINT_PTR_FMT              I32_FMT
  #define      PTR_FMTx      "%8.8" I32_FMT "x"
  #define      PTR_FMTX      "%8.8" I32_FMT "X"
 #endif
 
+#if defined(SIZEOF_SIZE_T) && SIZEOF_SIZE_T == 8
+  #define  SIZE_T_FMT              I64_FMT
+  #define  SIZE_T_FMTx    "%16.16" I64_FMT "x"
+  #define  SIZE_T_FMTX    "%16.16" I64_FMT "X"
+#else // !defined(SIZEOF_SIZE_T) || SIZEOF_SIZE_T != 8
+  #define  SIZE_T_FMT              I32_FMT
+  #define  SIZE_T_FMTx      "%8.8" I32_FMT "x"
+  #define  SIZE_T_FMTX      "%8.8" I32_FMT "X"
+#endif
+
 /*-------------------------------------------------------------------*/
 /* Socket stuff                                                      */
 /*-------------------------------------------------------------------*/
@@ -181,6 +222,7 @@ typedef struct bind_struct      bind_struct;      // Socket Device Ctl
 
 typedef struct TAPEMEDIA_HANDLER  TAPEMEDIA_HANDLER;  // (see tapedev.h)
 typedef struct TAPEAUTOLOADENTRY  TAPEAUTOLOADENTRY;  // (see tapedev.h)
+typedef struct TAMDIR             TAMDIR;             // (see tapedev.h)
 
 /*-------------------------------------------------------------------*/
 /* Device handler function prototypes                                */
