@@ -1,7 +1,7 @@
 /* IMPL.C       (c) Copyright Roger Bowler, 1999-2007                */
 /*              Hercules Initialization Module                       */
 
-// $Id: impl.c,v 1.123 2007/06/23 00:04:14 ivan Exp $
+// $Id: impl.c,v 1.129 2009/01/07 16:00:43 bernard Exp $
 
 /*-------------------------------------------------------------------*/
 /* This module initializes the Hercules S/370 or ESA/390 emulator.   */
@@ -12,6 +12,24 @@
 /*-------------------------------------------------------------------*/
 
 // $Log: impl.c,v $
+// Revision 1.129  2009/01/07 16:00:43  bernard
+// add msghldsec command
+//
+// Revision 1.128  2008/11/04 05:56:31  fish
+// Put ensure consistent create_thread ATTR usage change back in
+//
+// Revision 1.127  2008/11/03 15:31:54  rbowler
+// Back out consistent create_thread ATTR modification
+//
+// Revision 1.126  2008/10/24 13:47:11  fish
+// Fix RC file not processed nor HAO thread engaged if -d daemon mode
+//
+// Revision 1.125  2008/10/18 09:32:21  fish
+// Ensure consistent create_thread ATTR usage
+//
+// Revision 1.124  2007/12/10 23:12:02  gsmith
+// Tweaks to OPTION_MIPS_COUNTING processing
+//
 // Revision 1.123  2007/06/23 00:04:14  ivan
 // Update copyright notices to include current year (2007)
 //
@@ -121,7 +139,7 @@ int i;
             {
                 /* If the cpu is running but not executing
                    instructions then it must be malfunctioning */
-                if((sysblk.regs[i]->instcount == (U64)savecount[i])
+                if((INSTCOUNT(sysblk.regs[i]) == (U64)savecount[i])
                   && !HDC1(debug_watchdog_signal, sysblk.regs[i]) )
                 {
                     /* Send signal to looping CPU */
@@ -130,7 +148,7 @@ int i;
                 }
                 else
                     /* Save current instcount */
-                    savecount[i] = sysblk.regs[i]->instcount;
+                    savecount[i] = INSTCOUNT(sysblk.regs[i]);
             }
             else
                 /* mark savecount invalid as CPU not in running state */
@@ -270,6 +288,17 @@ TID     logcbtid;                       /* RC file thread identifier */
     /* Clear the system configuration block */
     memset (&sysblk, 0, sizeof(SYSBLK));
 
+#ifdef OPTION_MSGHLD
+    /* Set the default timeout value */
+    sysblk.keep_timeout_secs = 120;
+#endif
+
+    /* Initialize thread creation attributes so all of hercules
+       can use them at any time when they need to create_thread
+    */
+    initialize_detach_attr (DETACHED);
+    initialize_join_attr   (JOINABLE);
+
     /* Copy length for regs */
     sysblk.regs_copy_len = (int)((uintptr_t)&sysblk.dummyregs.regs_copy_end
                                - (uintptr_t)&sysblk.dummyregs);
@@ -477,7 +506,7 @@ TID     logcbtid;                       /* RC file thread identifier */
 
 #if !defined(NO_SIGABEND_HANDLER)
     /* Start the watchdog */
-    if ( create_thread (&sysblk.wdtid, &sysblk.detattr,
+    if ( create_thread (&sysblk.wdtid, DETACHED,
                         watchdog_thread, NULL, "watchdog_thread") )
     {
         fprintf (stderr,
@@ -491,7 +520,7 @@ TID     logcbtid;                       /* RC file thread identifier */
     if (sysblk.httpport)
     {
         /* Start the http server connection thread */
-        if ( create_thread (&sysblk.httptid, &sysblk.detattr,
+        if ( create_thread (&sysblk.httptid, DETACHED,
                             http_server, NULL, "http_server") )
         {
             fprintf (stderr,
@@ -505,7 +534,7 @@ TID     logcbtid;                       /* RC file thread identifier */
 #ifdef OPTION_SHARED_DEVICES
     /* Start the shared server */
     if (sysblk.shrdport)
-        if ( create_thread (&sysblk.shrdtid, &sysblk.detattr,
+        if ( create_thread (&sysblk.shrdtid, DETACHED,
                             shared_server, NULL, "shared_server") )
         {
             fprintf (stderr,
@@ -521,7 +550,7 @@ TID     logcbtid;                       /* RC file thread identifier */
 
         for (dev = sysblk.firstdev; dev != NULL; dev = dev->nextdev)
             if (dev->connecting)
-                if ( create_thread (&tid, &sysblk.detattr,
+                if ( create_thread (&tid, DETACHED,
                            *dev->hnd->init, dev, "device connecting thread")
                    )
                 {
@@ -534,14 +563,14 @@ TID     logcbtid;                       /* RC file thread identifier */
 #endif
 
     /* Start up the RC file processing thread */
-    create_thread(&rctid,&sysblk.detattr,
+    create_thread(&rctid,DETACHED,
                   process_rc_file,NULL,"process_rc_file");
 
     if(log_callback)
     {
         // 'herclin' called us. IT'S in charge. Create its requested
         // logmsg intercept callback function and return back to it.
-        create_thread(&logcbtid,&sysblk.detattr,
+        create_thread(&logcbtid,DETACHED,
                       log_do_callback,NULL,"log_do_callback");
         return(0);
     }
@@ -554,15 +583,23 @@ TID     logcbtid;                       /* RC file thread identifier */
     if(!sysblk.daemon_mode)
         panel_display ();
     else
+    {
 #if defined(OPTION_DYNAMIC_LOAD)
         if(daemon_task)
             daemon_task ();
         else
 #endif /* defined(OPTION_DYNAMIC_LOAD) */
+        {
+            /* Tell RC file and HAO threads they may now proceed */
+            sysblk.panel_init = 1;
+
+            /* Retrieve messages from logger and write to stderr */
             while (1)
                 if((msgcnt = log_read(&msgbuf, &msgnum, LOG_BLOCK)))
                     if(isatty(STDERR_FILENO))
                         fwrite(msgbuf,msgcnt,1,stderr);
+        }
+    }
 
     //  -----------------------------------------------------
     //      *** Hercules has been shutdown (PAST tense) ***
