@@ -1,9 +1,18 @@
 /* SOCKDEV.C    (c) Copyright Hercules development, 2003-2007        */
 /*              Socketdevice support                                 */
 
-// $Id: sockdev.c,v 1.26 2007/06/23 00:04:15 ivan Exp $
+// $Id: sockdev.c,v 1.29 2008/11/04 05:56:31 fish Exp $
 //
 // $Log: sockdev.c,v $
+// Revision 1.29  2008/11/04 05:56:31  fish
+// Put ensure consistent create_thread ATTR usage change back in
+//
+// Revision 1.28  2008/11/03 15:31:53  rbowler
+// Back out consistent create_thread ATTR modification
+//
+// Revision 1.27  2008/10/18 09:32:21  fish
+// Ensure consistent create_thread ATTR usage
+//
 // Revision 1.26  2007/06/23 00:04:15  ivan
 // Update copyright notices to include current year (2007)
 //
@@ -525,9 +534,7 @@ int bind_device (DEVBLK* dev, char* spec)
 
     if ( was_list_empty )
     {
-        ATTR joinable_attr;
-        initialize_join_attr( &joinable_attr );
-        if ( create_thread( &sysblk.socktid, &joinable_attr,
+        if ( create_thread( &sysblk.socktid, JOINABLE,
                             socket_thread, NULL, "socket_thread" ) )
             {
                 logmsg( _( "HHCSD023E Cannot create socketdevice thread: errno=%d: %s\n" ),
