@@ -4,7 +4,7 @@
 /* Interpretive Execution - (c) Copyright Jan Jaeger, 1999-2007      */
 /* z/Architecture support - (c) Copyright Jan Jaeger, 1999-2007      */
 
-// $Id: stack.c,v 1.84 2007/06/23 00:04:16 ivan Exp $
+// $Id: stack.c,v 1.89 2008/04/11 14:29:28 bernard Exp $
 
 /*-------------------------------------------------------------------*/
 /* This module implements the linkage stack functions of ESA/390     */
@@ -25,6 +25,31 @@
 /*-------------------------------------------------------------------*/
 
 // $Log: stack.c,v $
+// Revision 1.89  2008/04/11 14:29:28  bernard
+// Integrate regs->exrl into base Hercules code.
+//
+// Revision 1.88  2008/04/09 07:38:58  bernard
+// Allign to Rogers terminal ;-)
+//
+// Revision 1.87  2008/04/08 17:15:10  bernard
+// Added execute relative long instruction
+//
+// Revision 1.86  2008/03/01 00:00:34  ptl00
+// Fix TRAP in z/Arch mode
+//
+// Revision 1.85  2008/02/12 18:23:39  jj
+// 1. SPKA was missing protection check (PIC04) because
+//    AIA regs were not purged.
+//
+// 2. BASR with branch trace and PIC16, the pgm old was pointing
+//    2 bytes before the BASR.
+//
+// 3. TBEDR , TBDR using R1 as source, should be R2.
+//
+// 4. PR with page crossing stack (1st page invalid) and PSW real
+//    in stack, missed the PIC 11. Fixed by invoking abs_stck_addr
+//    for previous stack entry descriptor before doing the load_psw.
+//
 // Revision 1.84  2007/06/23 00:04:16  ivan
 // Update copyright notices to include current year (2007)
 //
@@ -335,18 +360,19 @@ int  i;
         }
 
     /* Load the Trap Control Block Address in gr15 */
-    regs->GR_L(15) = duct11 & DUCT11_TCBA;
+#if defined(FEATURE_ESAME)
+    if(regs->psw.amode64)
+        regs->GR(15) = duct11 & DUCT11_TCBA & 0x00000000FFFFFFFF;
+    else
+#endif /*defined(FEATURE_ESAME)*/
+        regs->GR_L(15) = duct11 & DUCT11_TCBA;
 
     /* Ensure psw.IA is set */
     SET_PSW_IA(regs);
 
     /* Set the Breaking Event Address Register */
-    SET_BEAR_REG(regs, regs->ip - ((trap_is_trap4 || regs->execflag) ? 4 : 2));
-
-    /* Set the Trap program address as a 31 bit instruction address */
-#if defined(FEATURE_ESAME)
-    regs->psw.amode64 = 0;
-#endif /*defined(FEATURE_ESAME)*/
+    SET_BEAR_REG(regs, regs->ip - 
+      (trap_is_trap4 ? 4 : regs->execflag ? regs->exrl ? 6 : 4 : 2));
     regs->psw.amode = 1;
     regs->psw.AMASK = AMASK31;
     UPD_PSW_IA(regs, trap_ia);
@@ -1395,6 +1421,15 @@ VADR    lsep;                           /* Virtual addr of entry desc.
 
 #endif /*defined(FEATURE_ESAME)*/
 
+    /* [5.12.4.4] Pass back the absolute address of the entry
+       descriptor of the preceding linkage stack entry.  The
+       next entry size field of this entry will be cleared on
+       successful completion of the PR instruction */
+    *lsedap = ARCH_DEP(abs_stack_addr) (lsep, regs, ACCTYPE_WRITE);
+
+    /* [5.12.4.5] Update CR15 to point to the previous entry */
+    regs->CR(15) = lsep & CR15_LSEA;
+
     /* Load new PSW using the bytes extracted from the stack entry */
     /* The rc will be checked by calling routine for PIC 06        */
     *rc = ARCH_DEP(load_psw) (regs, newpsw);
@@ -1408,15 +1443,6 @@ VADR    lsep;                           /* Virtual addr of entry desc.
     /* restore PER masks which could have been wiped out by load_psw */
     SET_IC_MASK(regs);
 
-    /* [5.12.4.4] Pass back the absolute address of the entry
-       descriptor of the preceding linkage stack entry.  The
-       next entry size field of this entry will be cleared on
-       successful completion of the PR instruction */
-    *lsedap = ARCH_DEP(abs_stack_addr) (lsep, regs, ACCTYPE_WRITE);
-
-    /* [5.12.4.5] Update CR15 to point to the previous entry */
-    regs->CR(15) = lsep & CR15_LSEA;
-
 #ifdef STACK_DEBUG
     logmsg (_("stack: CR15=" F_CREG "\n"), regs->CR(15));
 #endif /*STACK_DEBUG*/
