@@ -1,7 +1,7 @@
 /* TAPEMAP.C   (c) Copyright Jay Maynard, 2000-2007                  */
 /*              Map AWSTAPE format tape image                        */
 
-// $Id: tapemap.c,v 1.26 2007/06/23 00:04:18 ivan Exp $
+// $Id: tapemap.c,v 1.27 2008/11/04 04:50:46 fish Exp $
 
 /*-------------------------------------------------------------------*/
 /* This program reads an AWSTAPE format tape image file and produces */
@@ -9,6 +9,9 @@
 /*-------------------------------------------------------------------*/
 
 // $Log: tapemap.c,v $
+// Revision 1.27  2008/11/04 04:50:46  fish
+// Ensure consistent utility startup
+//
 // Revision 1.26  2007/06/23 00:04:18  ivan
 // Update copyright notices to include current year (2007)
 //
@@ -71,17 +74,7 @@ BYTE            labelrec[81];           /* Standard label (ASCIIZ)   */
 AWSTAPE_BLKHDR  awshdr;                 /* AWSTAPE block header      */
 char            pathname[MAX_PATH];     /* file path in host format  */
 
-    set_codepage(NULL);
-
-#ifdef EXTERNALGUI
-    if (argc >= 1 && strncmp(argv[argc-1],"EXTERNALGUI",11) == 0)
-    {
-        extgui = 1;
-        argc--;
-        setvbuf(stderr, NULL, _IONBF, 0);
-        setvbuf(stdout, NULL, _IONBF, 0);
-    }
-#endif /*EXTERNALGUI*/
+    INITIALIZE_UTILITY("tapemap");
 
     /* Display the program identification message */
     display_version (stderr, "Hercules tape map program ", FALSE);
